package controllers

import (
	"encoding/json"
	"hongze/hongze_mobile_admin/models/request/admin"
	admin3 "hongze/hongze_mobile_admin/models/response/admin"
	"hongze/hongze_mobile_admin/models/tables/h5_admin_session"
	"hongze/hongze_mobile_admin/services"
	"hongze/hongze_mobile_admin/utils"
	"time"
)

type AdminCommon struct {
	BaseAuth
}

// @Title 用户账号、密码登录接口
// @Description 用户账号、密码登录接口
// @Param	request	body admin.LoginReq true "type json string"
// @Success 200 {object} admin.LoginResp
// @router /login [post]
func (this *AdminCommon) Login() {
	var req admin.LoginReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		this.FailWithMessage("参数解析异常！", "参数解析失败，Err:"+err.Error())
		return
	}
	if req.Username == "" {
		this.FailWithMessage("请输入账号", "请输入账号")
		return
	}
	if req.Password == "" {
		this.FailWithMessage("请输入密码", "请输入密码")
		return
	}

	adminWx, err := services.BindWxUser(this.Session.OpenId, req.Username, req.Password, utils.WxPlatform)
	if err != nil {
		if err.Error() == utils.ErrNoRow() {
			this.FailWithMessage("登录失败,账号或密码错误", "登录失败,账号或密码错误")
			return
		}
		this.FailWithMessage("登录失败", "登录失败，Err:"+err.Error())
		return
	}
	err = h5_admin_session.UpdateSession(this.Session.SessionId, adminWx.AdminId, time.Now().AddDate(0, 0, 90))
	if err != nil {
		this.FailWithMessage("登录失败", "变更session信息失败，Err:"+err.Error())
		return
	}
	resp := admin3.LoginResp{
		RealName:     adminWx.RealName,
		AdminName:    adminWx.AdminName,
		RoleName:     adminWx.RoleName,
		RoleTypeCode: adminWx.RoleTypeCode,
		AdminId:      adminWx.AdminId,
	}

	resp.AdminId = adminWx.AdminId
	var productName string
	productId := services.GetProductId(adminWx.RoleTypeCode)

	if productId == 1 {
		productName = utils.COMPANY_PRODUCT_FICC_NAME
	} else if productId == 2 {
		productName = utils.COMPANY_PRODUCT_RAI_NAME
	} else {
		productName = "admin"
	}
	resp.ProductName = productName
	resp.Authority = adminWx.Authority
	this.OkDetailed(resp, "登录成功")
}

// @Title 用户退出登录接口
// @Description 用户退出登录接口
// @Success Ret=200 退出成功
// @router /logout [post]
func (this *AdminCommon) Logout() {
	adminWx := this.AdminWx
	err := services.UnbindWxUser(adminWx.OpenId)
	if err != nil {
		this.FailWithMessage("退出失败", "退出失败，Err:"+err.Error())
		return
	}
	err = h5_admin_session.UpdateSession(this.Session.SessionId, 0, time.Now().AddDate(0, 0, 10))
	if err != nil {
		this.FailWithMessage("退出失败", "变更session信息失败，Err:"+err.Error())
		return
	}
	this.OkWithMessage("退出成功")
}
