package controllers

import (
	"encoding/json"
	"fmt"
	"github.com/beego/beego/v2/server/web"
	"hongze/hongze_mobile_admin/utils"
	"net/url"
	"rdluck_tools/log"
)

var apiLog *log.Log

func init() {
	if utils.RunMode == "release" {
		logDir := `/data/rdlucklog/hongze_api`
		apiLog = log.Init("20060102.api", logDir)
	} else {
		apiLog = log.Init("20060102.api")
	}
}

//不需要授权的基类
type BaseCommon struct {
	web.Controller
	Response
}

type Response struct {
	Code   int         `json:"code"`
	Data   interface{} `json:"data"`
	Msg    string      `json:"msg"`
	ErrMsg string      `json:"errMsg";description:"给开发工程师看的错误信息"`
}

const (
	SUCCESS     = 200 //成功
	ERROR       = 400 //代表业务处理失败，前端同学需要做额外逻辑处理
	TOKEN_ERROR = 401 //代表token异常，用户需要重新静默授权，获取最新的token
	BIND_ERROR  = 403 //403代表用户需要进行绑定操作，需要跳转到输入账号密码绑定页面
	SIGN_ERROR  = 3   //签名异常
)

//返回数据
func (this BaseCommon) Result() {
	var content []byte
	var err error
	content, err = json.Marshal(this.Response)
	ip := this.Ctx.Input.IP()
	requestBody, err := url.QueryUnescape(string(this.Ctx.Input.RequestBody))
	if err != nil {
		fmt.Println("base auth url.QueryUnescape Err:", err.Error())
		requestBody = string(this.Ctx.Input.RequestBody)
	}
	apiLog.Println("请求地址：", this.Ctx.Input.URI(), "Authorization:", this.Ctx.Input.Header("Authorization"), "RequestBody：", requestBody, "ResponseBody", string(content), "IP：", ip)

	this.Controller.Data["json"] = this.Response
	this.Controller.ServeJSON()
	this.StopRun()
}

//没有任何信息的返回
func (this BaseCommon) Ok() {
	this.Response.Code = SUCCESS
	this.Response.Msg = "操作成功"
	this.Response.Data = map[string]interface{}{}
	this.Result()
}

func (this BaseCommon) OkWithMessage(message string) {
	this.Response.Code = SUCCESS
	this.Response.Msg = message
	this.Response.Data = map[string]interface{}{}
	this.Result()
}

func (this BaseCommon) OkWithData(data interface{}) {
	this.Response.Code = SUCCESS
	this.Response.Msg = "操作成功"
	this.Response.Data = data
	this.Result()
}

func (this BaseCommon) OkDetailed(data interface{}, message string) {
	this.Response.Code = SUCCESS
	this.Response.Msg = message
	this.Response.Data = data
	this.Result()
}

func (this BaseCommon) Fail() {
	this.Response.Code = ERROR
	this.Response.Msg = "操作失败"
	this.Response.Data = map[string]interface{}{}
	this.Result()
}

func (this BaseCommon) FailWithMessage(message, errMessage string) {
	this.Response.Code = ERROR
	this.Response.Msg = message
	this.Response.ErrMsg = errMessage
	this.Response.Data = map[string]interface{}{}
	this.Result()
}

//token异常
func (this BaseCommon) TokenError(data interface{}, message string) {
	this.Response.Code = TOKEN_ERROR
	this.Response.Msg = message
	this.Response.Data = data
	this.Result()
}

//token异常
func (this BaseCommon) TokenMsgError(message, errMessage string) {
	this.Response.Code = TOKEN_ERROR
	this.Response.Msg = message
	this.Response.ErrMsg = errMessage
	//this.Response.Data = data
	this.Result()
}

//账户绑定异常
func (this BaseCommon) BindError(data interface{}, message string) {
	this.Response.Code = BIND_ERROR
	this.Response.Msg = message
	this.Response.Data = data
	this.Result()
}

//账户绑定异常
func (this BaseCommon) BindMsgError(message, errMessage string) {
	this.Response.Code = BIND_ERROR
	this.Response.Msg = message
	this.Response.ErrMsg = errMessage
	//this.Response.Data = data
	this.Result()
}

//签名异常
func (this BaseCommon) SignError(message string) {
	this.Response.Code = SIGN_ERROR
	this.Response.Msg = message
	this.Response.Data = map[string]interface{}{}
	this.Result()
}

func (this BaseCommon) FailWithDetailed(code int, data interface{}, message string) {
	this.Response.Code = code
	this.Response.Msg = message
	this.Response.Data = data
	this.Result()
}
