package controllers

import (
	"encoding/json"
	sealReq "hongze/hongze_mobile_admin/models/request/seal"
	sealResp "hongze/hongze_mobile_admin/models/response/seal"
	"hongze/hongze_mobile_admin/models/tables/contract_approval"
	"hongze/hongze_mobile_admin/models/tables/contract_approval_record"
	"hongze/hongze_mobile_admin/models/tables/seal"
	sealService "hongze/hongze_mobile_admin/services/seal"
	"hongze/hongze_mobile_admin/utils"
	"rdluck_tools/paging"
	"strconv"
	"strings"
)

//用印审批模块
type SealApprovalCommon struct {
	BaseAuth
}

// @Title 审批单列表
// @Description 审批单列表接口
// @Param   Use   query   string  false       "用途，枚举值：销售合同, 渠道合同, 付款通知函, 招投标, 战略合作协议"
// @Param   ServiceType   query   string  false       "业务类型，枚举值："新签合同, 续约合同, 补充协议"
// @Param   SealType   query   string  false       "公章类型，枚举值：合同章, 公章, 法人章"
// @Param   Status   query   string  false       "合同状态，枚举值：'待审批','已审批','已驳回','已撤回'"
// @Param   UserId   query   string  false       "选择的申请人id"
// @Param   Keyword   query   string  false       "搜索关键字"
// @Param   ModifyStartTime   query   string  false       "服务更新时间的选择开始时间，格式：2021-05-23 00:00:00"
// @Param   ModifyEndTime   query   string  false       "服务更新时间的选择结束时间，格式：2021-05-26 23:59:59"
// @Success 200 {object} seal.SealApprovalListResp
// @router /list [get]
func (this *SealApprovalCommon) List() {
	sysUser := this.AdminWx

	//合同类型、产品类型、合同状态、更新时间、所选销售
	//关键字：合同编号、客户名称，社会信用码
	serviceType := this.GetString("ServiceType")
	use := this.GetString("Use")
	sealType := this.GetString("SealType")
	status := this.GetString("Status")
	productId, _ := this.GetInt("ProductId")
	sellerIds := this.GetString("SellerId")
	keyword := this.GetString("Keyword")
	modifyStartTime := this.GetString("ModifyStartTime")
	modifyEndTime := this.GetString("ModifyEndTime")

	childCondition := ""
	condition := ""
	childPars := make([]interface{}, 0)
	pars := make([]interface{}, 0)
	////如果不是超管或者合规，那么只能查看自己的合同
	//if sysUser.RoleTypeCode == utils.ROLE_TYPE_CODE_RAI_ADMIN { //权益管理员
	//	condition += ` AND c.product_id = ? `
	//	pars = append(pars, 2)
	//} else if sysUser.RoleTypeCode == utils.ROLE_TYPE_CODE_FICC_ADMIN { //ficc管理员
	//	condition += ` AND c.product_id = ? `
	//	pars = append(pars, 1)
	//} else {
	//	if sysUser.RoleTypeCode != utils.ROLE_TYPE_CODE_COMPLIANCE && sysUser.RoleTypeCode != utils.ROLE_TYPE_CODE_ADMIN {
	//		condition += ` AND c.seller_id = ? `
	//		pars = append(pars, sysUser.AdminId)
	//	}
	//}

	condition += ` AND (c.user_id = ? or d.approve_user_id = ?)`
	pars = append(pars, sysUser.AdminId, sysUser.AdminId)

	//合同类型、、更新时间、所选销售
	//关键字：合同编号、客户名称，社会信用码
	if use != "" {
		condition += ` AND c.use = ? `
		pars = append(pars, use)
	}
	if serviceType != "" {
		condition += ` AND c.service_type = ? `
		pars = append(pars, serviceType)
	}
	if sealType != "" {
		condition += ` AND c.seal_type = ? `
		pars = append(pars, sealType)
	}
	//审批状态
	if status != "" {
		if status == "已审批" {
			condition += ` AND c.status in ("已审批","已驳回")  `
		} else {
			condition += ` AND c.status = ? `
			if status == "处理中" {
				pars = append(pars, "待审批")
			} else {
				pars = append(pars, status)
			}
		}
	} else {
		//childCondition += ` AND status != "已撤回" `

		condition += ` AND c.status not in ("已撤回","待提交")  `
	}
	//产品类型
	if productId > 0 {
		condition += ` AND c.product_id = ? `
		pars = append(pars, productId)
	}
	//所选销售
	if sellerIds != "" {
		condition += ` AND c.user_id IN (` + sellerIds + `) `
	}
	//更新开始时间

	//更新开始时间
	if modifyStartTime != "" {
		condition += ` AND a.modify_time >= ? `
		pars = append(pars, modifyStartTime)
	}
	//更新结束时间
	if modifyEndTime != "" {
		condition += ` AND a.modify_time <= ? `
		pars = append(pars, modifyEndTime)
	}
	//关键字
	if keyword != "" {
		condition += ` AND (c.contract_code LIKE '%` + keyword + `%' OR c.company_name LIKE '%` + keyword + `%' OR c.credit_code LIKE '%` + keyword + `%' ) `
	}

	pageSize, _ := this.GetInt("PageSize")
	currentIndex, _ := this.GetInt("CurrentIndex")

	var startSize int
	if pageSize <= 0 {
		pageSize = utils.PageSize20
	}
	if currentIndex <= 0 {
		currentIndex = 1
	}
	startSize = paging.StartIndex(currentIndex, pageSize)

	var total int
	var list []*contract_approval.SealApprovalList

	//join表字段
	joinCondition := " and a.curr_node_id=d.node_id"
	if status == "处理中" {
		joinCondition = " and a.curr_node_id!=d.node_id"
	}
	if status != "待审批" && status != "处理中" {
		tmpTotal, err := contract_approval.GetSealApprovalListCountV2(childCondition, condition, childPars, pars)
		if err != nil {
			this.FailWithMessage("获取失败", "获取数据总数失败，Err:"+err.Error())
			return
		}

		tmpList, err := contract_approval.GetSealApprovalListV2(childCondition, condition, childPars, pars, startSize, pageSize)
		if err != nil {
			this.FailWithMessage("获取审批列表失败！", "获取审批列表失败，Err:"+err.Error())
			return
		}
		total = tmpTotal
		list = tmpList
	} else {
		tmpTotal, err := contract_approval.GetSealApprovalListCount(childCondition, condition, joinCondition, childPars, pars)
		if err != nil {
			this.FailWithMessage("获取失败", "获取数据总数失败，Err:"+err.Error())
			return
		}

		tmpList, err := contract_approval.GetSealApprovalList(childCondition, condition, joinCondition, childPars, pars, startSize, pageSize)
		if err != nil {
			this.FailWithMessage("获取审批列表失败！", "获取审批列表失败，Err:"+err.Error())
			return
		}
		total = tmpTotal
		list = tmpList
	}

	if len(list) > 0 {
		contractApprovalIdSlice := make([]string, 0)
		for i := 0; i < len(list); i++ {
			contractApprovalIdSlice = append(contractApprovalIdSlice, strconv.Itoa(list[i].ContractApprovalId))
		}
		contractApprovalIdStr := strings.Join(contractApprovalIdSlice, ",")

		//获取审批流列表数据
		contractApprovalRecordList, err := contract_approval_record.GetContractApprovalRecordList(contractApprovalIdStr, sysUser.AdminId)
		if err != nil {
			this.FailWithMessage("获取审批列表失败！", "获取审批列表失败，Err:"+err.Error())
			return
		}
		contractApprovalRecordMap := make(map[int]*contract_approval_record.ContractApprovalRecord)
		for i := 0; i < len(contractApprovalRecordList); i++ {
			contractApprovalRecordMap[contractApprovalRecordList[i].ContractApprovalId] = contractApprovalRecordList[i]
		}

		for i := 0; i < len(list); i++ {
			item := list[i]
			list[i].CreateTimeStr = item.CreateTime.Format(utils.FormatDateTime)
			list[i].ModifyTimeStr = item.ModifyTime.Format(utils.FormatDateTime)

			if item.Status == "已审批" || item.Status == "已驳回" {
				list[i].ApproveTime = item.ModifyTime
				list[i].ApproveTimeStr = item.ModifyTime.Format(utils.FormatDateTime)
			}

			//list[i].ContractInfo = contractDetail
			//校验最新审批流数据
			//if contractApprovalRecord, has := contractApprovalRecordMap[item.ContractApprovalId]; has {
			//	//审批流id
			//	list[i].ContractApprovalRecordId = contractApprovalRecord.ContractApprovalRecordId
			//	//审批时间
			//	list[i].ApproveTime = contractApprovalRecord.ApproveTime
			//	list[i].ApproveTimeStr = contractApprovalRecord.ApproveTime.Format(utils.FormatDateTime)
			//	//判断是否有操作权限
			//	list[i].Status = contractApprovalRecord.Status
			//
			//	//判断是否处于待审批状态
			//	//if contractApprovalRecord.Status == "待审批" {
			//	//	//如果不是审批者，那么不能审批操作
			//	//	if contractApprovalRecord.NodeType != "check" {
			//	//		continue
			//	//	}
			//	//	//如果没有指定审批人的话
			//	//	if contractApprovalRecord.ApproveUserId <= 0 {
			//	//		//判断是否与审批角色一致，如果一致，那么允许操作审批
			//	//		if contractApprovalRecord.ApproveRoleTypeCode == sysUser.RoleTypeCode {
			//	//			list[i].CanApprovalButton = true
			//	//		}
			//	//	} else if contractApprovalRecord.ApproveUserId == sysUser.AdminId {
			//	//		//如果指定审批人就是自己的话，那么操作审批
			//	//		list[i].CanApprovalButton = true
			//	//	}
			//	//}
			//}

			//合同状态（突然说要改成展示合同状态，那就根据业务需求，改成返回：合同状态咯）
			//list[i].Status = item.SealStatus
			if item.Status == "待审批" && status == "处理中" {
				list[i].Status = "处理中"
			}
		}
	}

	page := paging.GetPaging(currentIndex, pageSize, total)
	this.OkDetailed(sealResp.SealApprovalListResp{
		List:   list,
		Paging: page,
	}, "获取成功")
}

// @Title 审批单详情
// @Description 审批单详情接口
// @Param   ContractApprovalId   query   int  false       "审批单id"
// @Param   ContractApprovalRecordId   query   int  false       "审批流程单id"
// @Success 200 {object} seal.SealDetailResp
// @router /detail [get]
func (this *SealApprovalCommon) Detail() {
	//审批流程单id
	contractApprovalRecordId, _ := this.GetInt("ContractApprovalRecordId")
	if contractApprovalRecordId > 0 {
		this.detailByApprovalRecordId(contractApprovalRecordId)
		return
	}

	//审批单id
	contractApprovalId, _ := this.GetInt("ContractApprovalId")

	//审批单详情
	contractApprovalInfo, err := contract_approval.GetContractApprovalById(contractApprovalId)
	if err != nil {
		this.FailWithMessage("获取审批单失败！", "获取审批单失败，Err:"+err.Error())
		return
	}

	//用印详情
	sealInfo, flowNodeListResp, opButton, err := sealService.GetSealDetailBySealId(contractApprovalInfo.ContractId, this.AdminWx)
	if err != nil {
		this.FailWithMessage("获取详情失败", "获取详情失败,Err:"+err.Error())
		return
	}
	resp := sealResp.SealDetailResp{
		SealDetail:   sealInfo,
		FlowNodeList: flowNodeListResp,
		OpButton:     opButton,
	}
	this.OkDetailed(resp, "获取成功")
}

func (this *SealApprovalCommon) detailByApprovalRecordId(approvalRecordId int) {
	approvalRecordInfo, err := contract_approval_record.GetContractApprovalRecordById(approvalRecordId)
	if err != nil {
		this.FailWithMessage("获取审批流程单失败", "获取审批流程单失败，Err:"+err.Error())
		return
	}

	//审批单详情
	contractApprovalInfo, err := contract_approval.GetContractApprovalById(approvalRecordInfo.ContractApprovalId)
	if err != nil {
		this.FailWithMessage("获取审批单失败", "获取审批单失败，Err:"+err.Error())
		return
	}

	//用印详情
	sealInfo, flowNodeListResp, opButton, err := sealService.GetSealDetailBySealId(contractApprovalInfo.ContractId, this.AdminWx)
	if err != nil {
		this.FailWithMessage("获取详情失败", "获取详情失败,Err:"+err.Error())
		return
	}
	resp := sealResp.SealDetailResp{
		SealDetail:   sealInfo,
		FlowNodeList: flowNodeListResp,
		OpButton:     opButton,
	}

	this.OkDetailed(resp, "获取成功")
	return
}

// @Title 驳回审批
// @Description 驳回审批接口
// @Param	request	body seal.RejectReq true "type json string"
// @Success Ret=200 驳回成功
// @router /reject [post]
func (this *SealApprovalCommon) Reject() {
	var req sealReq.RejectReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		this.FailWithMessage("参数解析异常！", "参数解析失败，Err:"+err.Error())
		return
	}
	//用印id编号
	sealId := req.SealId
	if sealId <= 0 {
		this.FailWithMessage("请传入用印单编号！", "请传入用印单编号！")
		return
	}

	//数据校验
	sealInfo, approvalInfo, approvalRecord, err := sealService.CheckApproveAuth(sealId, this.AdminWx)
	if err != nil {
		this.FailWithMessage("驳回审批失败！", "驳回审批失败，Err:"+err.Error())
		return
	}
	err = sealService.Reject(sealInfo, approvalInfo, approvalRecord, this.AdminWx, req.Remark)
	if err != nil {
		this.FailWithMessage("驳回审批失败！", "驳回审批失败，Err:"+err.Error())
		return
	}
	this.OkWithMessage("驳回成功")
}

// @Title 处理审批单
// @Description 处理审批单
// @Param	request	body seal.ApprovedReq true "type json string"
// @Success Ret=200 申请成功
// @router /approved [post]
func (this *SealApprovalCommon) Approved() {
	var req sealReq.ApprovedReq
	err := json.Unmarshal(this.Ctx.Input.RequestBody, &req)
	if err != nil {
		this.FailWithMessage("参数解析异常！", "参数解析失败，Err:"+err.Error())
		return
	}
	//用印编号id编号
	sealId := req.SealId
	if sealId <= 0 {
		this.FailWithMessage("请传入用印单编号！", "请传入用印单编号！")
		return
	}

	//数据校验
	sealInfo, approvalInfo, approvalRecord, err := sealService.CheckApproveAuth(sealId, this.AdminWx)
	if err != nil {
		this.FailWithMessage("驳回审批失败！", "驳回审批失败，Err:"+err.Error())
		return
	}
	err = sealService.Approved(sealInfo, approvalInfo, approvalRecord, this.AdminWx, req.Remark)
	if err != nil {
		this.FailWithMessage("审核失败！", "审核失败，Err:"+err.Error())
		return
	}
	this.OkWithMessage("审核通过")
}

// @Title 撤回审批申请
// @Description 撤回审批申请接口
// @Param   SealId   query   int  true       "用印id编号"
// @Success Ret=200 撤回申请成功
// @router /cancelApply [get]
func (this *SealApprovalCommon) CancelApply() {
	//用印id编号
	sealId, err := this.GetInt("SealId")
	if err != nil {
		this.FailWithMessage("获取审批流编号失败", "获取审批流编号失败，err："+err.Error())
		return
	}

	//获取用印信息
	sealInfo, err := seal.GetSealInfoById(sealId)
	if err != nil {
		this.FailWithMessage("获取审批流信息失败", "获取审批流信息失败，err："+err.Error())
		return
	}
	err = sealService.CancelApply(sealInfo, this.AdminWx.AdminId)
	if err != nil {
		this.FailWithMessage("撤回申请失败！", "撤回申请失败，Err:"+err.Error())
		return
	}
	this.OkWithMessage("撤回申请成功")
}
